import { Component } from '@angular/core';
import {FormGroup, FormsModule, NonNullableFormBuilder, ReactiveFormsModule, Validators} from "@angular/forms";
import {NzButtonComponent} from "ng-zorro-antd/button";
import {NzColDirective, NzRowDirective} from "ng-zorro-antd/grid";
import {NzFormControlComponent, NzFormDirective, NzFormItemComponent, NzFormLabelComponent} from "ng-zorro-antd/form";
import {NzInputDirective} from "ng-zorro-antd/input";
import {DatePipe, NgForOf, NgIf, NgStyle} from "@angular/common";
import {NzOptionComponent, NzSelectComponent} from "ng-zorro-antd/select";
import {GoodsTypeData} from "../../../model/goods.interface";
import {NzModalModule} from "ng-zorro-antd/modal";
import {DictionaryPipe} from "../../../pipes/common/dictionary.pipe";
import {NzUploadComponent, NzUploadFile} from "ng-zorro-antd/upload";
import {NzIconDirective} from "ng-zorro-antd/icon";
import {NzMessageService} from "ng-zorro-antd/message";
import {environment} from "../../../../environments/environment";
import {CommonService} from "../../../services/common/common.service";
import {GoodsService} from "../../../services/goods/goods.service";
import {NzTableComponent, NzTableModule, NzTdAddOnComponent} from "ng-zorro-antd/table";
import {NzImageDirective , NzImageModule} from "ng-zorro-antd/image";
import {fallbackImg} from "../../../data/goods/goods.namespace";
import {NzTreeSelectComponent} from "ng-zorro-antd/tree-select";
import {NzDividerComponent} from "ng-zorro-antd/divider";
import {it} from "node:test";
import {NzPopconfirmDirective} from "ng-zorro-antd/popconfirm";

const getBase64 = (file: File): Promise<string | ArrayBuffer | null> =>
  new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = () => resolve(reader.result);
    reader.onerror = error => reject(error);
  });

@Component({
  selector: 'app-goods-type',
  standalone: true,
  imports: [
    FormsModule,
    NzButtonComponent,
    NzColDirective,
    NzFormControlComponent,
    NzFormDirective,
    NzFormItemComponent,
    NzFormLabelComponent,
    NzInputDirective,
    NzRowDirective,
    ReactiveFormsModule,
    NgForOf,
    NzOptionComponent,
    NzSelectComponent,
    NzModalModule,
    NgIf,
    NzUploadComponent,
    NzIconDirective,
    NzImageModule,
    NgStyle,
    NzTableComponent,
    NzTdAddOnComponent,
    NzTableModule,
    DictionaryPipe,
    DatePipe,
    NzImageDirective,
    NzTreeSelectComponent,
    NzDividerComponent,
    NzPopconfirmDirective
  ],
  templateUrl: './goods-type.component.html',
  styleUrl: './goods-type.component.less'
})
export class GoodsTypeComponent {

  // 编辑产品类型表单
  editGoodSTypeForm: FormGroup;
  // 列表数据
  listOfMapData: GoodsTypeData[] = [];
  // 列表展示数据
  mapOfExpandedData: { [id: string]: GoodsTypeData[] } = {};

  // 编辑弹出框
  isVisibleEdit = false;
  // 商品類型數據字典
  goodsTypeArray;
  // 上传等待
  loading = false;
  // 图片
  imgUrl: NzUploadFile[] = []
  baseUrl = environment.baseUrl;
  imageUrl = environment.imageUrl;

  // 展示图片
  previewImage: string | undefined = '';
  // 上传是否展示
  previewVisible = false;
  nodes = [];

  constructor(
    private fb: NonNullableFormBuilder,
    private commonService: CommonService,
    private goodsType: GoodsService,
    private msg: NzMessageService
  ) {

    this.getRequest();
    // 菜单类型
    this.goodsTypeArray = new DictionaryPipe().getDictionaryList('GOODS_TYPE');
    // 初始化
    this.editGoodSTypeForm = this.fb.group({
      title: ['' , [Validators.required]],
      parentId: [''],
      id: [null],
      businessType: ['' ,[Validators.required]],
      imgUrl: [''],
    });

  }

  getNode(array: any): any {
    array.forEach((item: any) => {
      if (item.children) {
        this.getNode(item.children);
      }else {
        item["isLeaf"] = true;
      }
      item["key"] = item.id;

    });
    this.nodes = array;
  }

  // 关闭
  collapse(array: GoodsTypeData[], data: GoodsTypeData, $event: boolean): void {
    if (!$event) {
      if (data.children) {
        data.children.forEach(d => {
          const target = array.find(a => a.id === d.id)!;
          target.expand = false;
          this.collapse(array, target, false);
        });
      } else {
        return;
      }
    }
  }

  // 配置数据
  convertTreeToList(root: GoodsTypeData): GoodsTypeData[] {
    const stack: GoodsTypeData[] = [];
    const array: GoodsTypeData[] = [];
    const hashMap = {};
    stack.push({ ...root, level: 0, expand: false });
    while (stack.length !== 0) {
      const node = stack.pop()!;
      this.visitNode(node, hashMap, array);
      if (node.children) {
        for (let i = node.children.length - 1; i >= 0; i--) {
          stack.push({ ...node.children[i], level: node.level! + 1, expand: false, parent: node });
        }
      }
    }
    return array;
  }



  // 插入数据
  visitNode(node: GoodsTypeData, hashMap: { [key: string]: boolean }, array: GoodsTypeData[]): void {
    if (!hashMap[node.id]) {
      hashMap[node.id] = true;
      array.push(node);
    }
  }


  // 搜索表单提交
  getRequest(): void {
    this.loading = true;
    this.goodsType.getList({time: new Date().getTime()}, (data: any) => {
      if (data['return_code'] === '000000') {
        this.listOfMapData = data['return_data'];
        this.getNode(this.listOfMapData);
        this.listOfMapData.forEach(item => {
          this.mapOfExpandedData[item.id] = this.convertTreeToList(item);
        });
      } else {
        this.msg.error(data['return_msg']);
      }
      this.loading = false;
    });
  }

  // 唤醒编辑表单
  showEditGoodsType(item?: GoodsTypeData): void {
    if (item != null) {
      item.businessType = String(item.businessType);
      this.editGoodSTypeForm.patchValue(item);
      if (!this.commonService.whetherStringIsNull(item.imgUrl)) {
        this.imgUrl = this.commonService.stitchImg(item.imgUrl);
      }
    } else {
      this.editGoodSTypeForm.reset();
    }
    this.isVisibleEdit = true;
  }
  // 编辑表单提交
  handleEdit(): void {
    if (this.editGoodSTypeForm.valid) {
      if (this.imgUrl.length !== 0) {
        this.editGoodSTypeForm.value.imgUrl = this.commonService.imgList(this.imgUrl);
      }
      this.goodsType.editGoodsType(this.editGoodSTypeForm.value , (data: any) => {
        if (data['return_code'] === '000000') {
          this.msg.success("成功！");
          this.isVisibleEdit = false;
          this.getRequest();
        } else {
          this.msg.error(data['return_msg']);
        }

      });
    } else {
      Object.values(this.editGoodSTypeForm.controls).forEach(control => {
        if (control.invalid) {
          control.markAsDirty();
          control.updateValueAndValidity({ onlySelf: true });
        }
      });
    }

  }

  // 删除
  delete(id: number): void {
    this.goodsType.delete(id , (data: any) => {
      if (data['return_code'] === '000000') {
        this.msg.success("成功！");
        this.getRequest();
      } else {
        this.msg.error(data['return_msg']);
      }
    });
  }

  // 上传操作
  handlePreview = async (file: NzUploadFile): Promise<void> => {
    console.log("数据" , file);
    if (!file.url && !file['preview']) {
      file['preview'] = await getBase64(file.originFileObj!);
    }
    this.previewImage = file.url || file['preview'];
    this.previewVisible = true;
  };


  protected readonly fallbackImg = fallbackImg;
}
